import json
import os
from typing import Any, Callable, Tuple

from consul import Consul


def load_config_from_consul(  # pylint: disable=too-many-arguments
        host: str,
        port: int,
        token: str,
        key: str,
        *other_keys: Tuple[str],
        deserializer: Callable = json.loads,
        **kwargs,
) -> dict:
    """
    Load config from a ``Consul`` server.

    ::

      <path-to-app-config>
          |___LOG_LEVEL=INFO
          |___DEBUG=false

    Suppose you have created config of your app on the path
    ``path-to-app-config`` on ``Consul`` server.

    If passed multiple keys for configs
    will merge them with priority from first mension to last mension.

    :param str host: host of a ``Consul`` server
    :param int port: port of a ``Consul`` server
    :param str token: access key to a ``Consul`` server
    :param str key: key path to a main config
    :param Tuple[str] other_keys: other paths to configs that should be merged
      with a main config
    :param Callable deserializer:
    """
    server = Consul(host, port, token, **kwargs)
    main_config = _get_config_for_keys_from_consul(
        server, f'{key.strip("/")}/', deserializer=deserializer,
    )
    other_configs = list()
    if other_keys:
        other_configs = [
            _get_config_for_keys_from_consul(
                server, f'{other_key.strip("/")}/', deserializer=deserializer,
            ) for other_key in other_keys
        ]

    configs = dict()
    for config in [main_config, *other_configs][::-1]:
        configs.update(config)
    return configs


def _get_config_for_keys_from_consul(
        server: Consul,
        key: str,
        deserializer: Callable,
) -> dict:
    _, raw_configs = server.kv.get(key, recurse=True)
    raw_configs = raw_configs[1:] if raw_configs else list()
    return {
        config['Key'].lstrip(key): deserializer(config['Value'])
        for config in raw_configs
    }


def _serialize_value_to_json(value: Any) -> str:
    return json.dumps(value) if not isinstance(value, (str)) else value


def _set_environment_variables(
        config: dict,
        prefix: str = '',
        serializer: Callable = _serialize_value_to_json,
) -> None:
    for key, value in config.items():
        os.environ[f'{prefix}{key}'] = serializer(value)



import json
import os
import sys
from pprint import pformat

from rconfig import load_config_from_consul
from rconfig.utils import to_bash


try:
    import click
except ImportError:
    sys.stderr.write('Missing cli packages.\nRun pip install "rconfig[cli]"')


@click.group()
@click.option(
    '-h',
    '--host',
    required=True,
    default=os.environ.get('RCONFIG_CONSUL_HOST'),
    show_default=True,
    help='Host of a consul server',
)
@click.option(
    '-a',
    '--access',
    required=True,
    default=os.environ.get('RCONFIG_CONSUL_ACCESS_KEY'),
    show_default=True,
    help='Access key for a consul server',
)
@click.option(
    '-p',
    '--port',
    type=click.INT,
    default=os.environ.get('RCONFIG_CONSUL_PORT', 8500),
    show_default=True,
    help='Port of consul server',
)
@click.option('-k', '--key', required=True, multiple=True, help='Consul key')
@click.pass_context
def cli(ctx, host, port, access, key):
    config = load_config_from_consul(host, port, access, *key)
    ctx.obj = dict()
    ctx.obj['CONFIG'] = config


@cli.command()
@click.pass_context
def list(ctx):  # pylint: disable=redefined-builtin
    "Show all config for given keys"
    if ctx.obj['CONFIG']:
        click.echo(pformat(ctx.obj['CONFIG']))
    else:
        click.echo('No config for given keys')
        sys.exit(1)


@cli.command()
@click.pass_context
@click.option('--prefix', default='', help='Prefix for environment keys')
@click.option(
    '-f',
    '--format',
    type=click.Choice(['json', 'bash'], case_sensitive=False),
    default='bash',
    show_default=True,
    help='Format of exported data',
)
def export(ctx, prefix, format):  # pylint: disable=R0913,W0622
    "Print out bash command export for all found config"
    if format == 'bash':
        envs = to_bash(ctx.obj['CONFIG'], prefix=prefix)
    else:
        envs = json.dumps(ctx.obj['CONFIG'])
    if not envs:
        sys.exit(1)
    click.echo(envs)


if __name__ == '__main__':
    cli()  # pylint: disable=no-value-for-parameter



from enum import Enum
from typing import Callable, Optional

from rconfig import _serialize_value_to_json


class ScriptFormat(Enum):
    inline: str = 'inline'
    new_line: str = 'new_line'


def to_bash(
        data: dict,
        format: ScriptFormat = ScriptFormat.inline,  # pylint: disable=W0622
        serializer: Callable = _serialize_value_to_json,
        prefix: str = '',
) -> Optional[str]:
    inline = ScriptFormat.inline == format
    bash = str()
    for key, value in data.items():
        line = f'{prefix}{key}={serializer(value)!r}'
        bash += f' {line}' if inline else f'export {line}\n'
    if not bash:
        return None
    return f'export {bash}' if inline else bash.rstrip('\n')
